/*
 * ZoeSoundGenOff.c
 *
 * Created: 22.04.2018 14:44:14
 * Author : chris
 */ 

#include <avr/io.h>
#include <stdint.h>
#include <stdbool.h>
#include <avr/interrupt.h>
#define F_CPU 16000
#include <util/delay.h>
#include <avr/wdt.h>
#include <avr/eeprom.h>

void SetupIO();
bool CheckButtonPressed();
void SetButtonPullup(bool state);
void SetVirtualButton(bool state);
void DoVirtualButtonCycle();
bool CheckLED();

bool button_pressed = false;
bool _button_pressed = false;

bool desired_state;
uint8_t eeprom_val;

void main()
{
	SetupIO();
	
	eeprom_val = eeprom_read_byte(0);
	
	desired_state = eeprom_val & 0b00000001;
	
	_delay_ms(500);
	
	while(true)
	{
		if (CheckLED() != desired_state)
		{
			DoVirtualButtonCycle();
			_delay_ms(500);
		}
		
		_button_pressed = button_pressed;
		button_pressed = CheckButtonPressed();
		
		if (button_pressed && !_button_pressed) // check for 'rising edge'
		{
			desired_state = !desired_state;
			eeprom_write_byte(0, desired_state);
		}
	}
}

void SetupIO()
{
	PORTA = 0;
	PORTB = 0;
	
	DDRA = 0b10111110;
	DDRB = 0b01111000;
}

bool CheckButtonPressed()
{
	bool button_pressed;
	SetButtonPullup(true);
	_delay_ms(20);
	button_pressed = ((PINA & 0b00000001) == 0b00000000);
	SetButtonPullup(false);
	
	return button_pressed;
}

void SetButtonPullup(bool state)
{
	if (state)
		PORTA |= 0b00000001;
	else
		PORTA &= 0b11111110;
}

void SetVirtualButton(bool state)
{
	if (state)
		PORTA |= 0b10000000;
	else
		PORTA &= 0b01111111;
}

bool CheckLED()
{
	return ((PINA & 0b01000000) == 0b01000000);
}


void DoVirtualButtonCycle()
{
	SetVirtualButton(true);
	_delay_ms(500);
	SetVirtualButton(false);
	_delay_ms(500);
}
